//3d vector calculations
//cartesian coordinates x,y,z are supposed
//(c)V.Glazkov, 2016

#ifndef _VECTOR_H
	#define _VECTOR_H


	#include<math.h>

	typedef struct _vector3d
		{
		double x;
		double y;
		double z;
		}vector3d;

//Functions declarations
	//standard vector algebra
	vector3d operator+(vector3d v1, vector3d v2);//sum vectors
	vector3d operator-(vector3d v1, vector3d v2);//subtract vectors
	vector3d operator*(vector3d v1, vector3d v2);//vector product
	vector3d operator*(double scale, vector3d v1);//multiply by scale
	vector3d operator/(vector3d v1, double c);//divide by scale
	double scalar(vector3d v1, vector3d v2);//scalar product
	double mod(vector3d v);//vector length
	double angle(vector3d v1, vector3d v2);//angle (grad.) between vectors
	//creation etc.
	vector3d makevector(double x, double y, double z);//create vector from cartesian coordinates
	vector3d makeunitvector(double theta, double phi);//create unit vector from polar coordinates
	double component(vector3d vec, int i);//get i-th(1=x,2=y,3=z) coordinate
	vector3d set_component(vector3d vec, int i, double val);//set i-th(1=x,2=y,3=z) coordinate to the specified value



//Functions definitions
	vector3d operator+(vector3d v1, vector3d v2)
		{
		vector3d v;

		v.x=v1.x+v2.x;
		v.y=v1.y+v2.y;
		v.z=v1.z+v2.z;

		return (v);
		}
	vector3d operator-(vector3d v1, vector3d v2)
		{
		vector3d v;

		v.x=v1.x-v2.x;
		v.y=v1.y-v2.y;
		v.z=v1.z-v2.z;

		return (v);
		}
	vector3d operator*(vector3d v1, vector3d v2)
		{
		vector3d v;

		v.x=v1.y*v2.z-v1.z*v2.y;
		v.y=-(v1.x*v2.z-v1.z*v2.x);
		v.z=v1.x*v2.y-v1.y*v2.x;

		return (v);
		}

	vector3d operator*(double scale, vector3d v1)
		{
		vector3d v;

		v.x=scale*v1.x;
		v.y=scale*v1.y;
		v.z=scale*v1.z;

		return (v);
		}
	vector3d operator/(vector3d v1, double c)
		{
		vector3d v;

		v=(1./c)*v1;

		return (v);
		}

	double scalar(vector3d v1, vector3d v2)
		{
		return(v1.x*v2.x+v1.y*v2.y+v1.z*v2.z);
		}

	double mod(vector3d v)
		{
		return(sqrt(scalar(v,v)));
		}

	double angle(vector3d v1, vector3d v2)
		{
		double tmp;

		if(mod(v1)==0||mod(v2)==0) return 0;
		tmp=scalar(v1,v2)/mod(v1)/mod(v2);
		tmp=acos(tmp)*180.0/M_PI;

		return(tmp);
		}



	vector3d makevector(double x, double y, double z)
		{
		vector3d v;
		v.x=x;
		v.y=y;
		v.z=z;

		return(v);
		}

	vector3d makeunitvector(double theta, double phi)
		{
		vector3d v;
		v.z=cos(theta);
		v.x=sin(theta)*cos(phi);
		v.y=sin(theta)*sin(phi);

		return(v);
		}

	double component(vector3d vec, int i)
		{
		double ans;

		switch(i)
			{
			case 1://x
				ans=vec.x;
				break;
			case 2://y
				ans=vec.y;
				break;
			case 3://z
				ans=vec.z;
				break;
			default://strange
				ans=0.;
				break;
			}

		return(ans);
		}
		
	vector3d set_component(vector3d vec, int i, double val)
		{
		vector3d vecout;

		switch(i)
			{
			case 1://x
				vecout=makevector(val,vec.y,vec.z);
				break;
			case 2://y
				vecout=makevector(vec.x,val,vec.z);
				break;
			case 3://z
				vecout=makevector(vec.x,vec.y,val);
				break;
			default://strange
				vecout=makevector(0.,0.,0.);
				break;
			}
		
		return(vecout);				
		}
#endif
